/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

public class Preferences implements WindowListener,ItemListener
{
	private static final String PREFS_FILE=
	  System.getProperty("user.home")+File.separator+"pilrcedit.prefs";
	public static boolean jdk11=System.getProperty("java.version").startsWith("1.1");
  public static String newLine=System.getProperty("line.separator");
  private static Image icon;
	private static Window main;
	private static Hashtable prefs=new Hashtable();
  private static ChangeListeners listeners=new ChangeListeners();
	private static Hashtable windows=new Hashtable();
	private static Vector visible=new Vector();
	private static Hashtable viewMenu=new Hashtable();
  private static Preferences listener=new Preferences();
  private static long deactivatedTime;
  private static GeneralDialog dialog;

  private Preferences()
  {
	}

  public void itemStateChanged(ItemEvent e)
  {
		CheckboxMenuItem cmi=(CheckboxMenuItem)e.getSource();
		Window w=(Window)windows.get(cmi.getLabel());
		w.setVisible(cmi.getState());
	}

	/**
	 * Invoked when a window has been opened.
	 */
	public void windowOpened(WindowEvent e)
	{
		visible.addElement(e.getSource());
		CheckboxMenuItem cmi=(CheckboxMenuItem)viewMenu.get(e.getSource());
		if (cmi!=null)
		  cmi.setState(true);
	}

	/**
	 * Invoked when a window is in the process of being closed.
	 * The close operation can be overridden at this point.
	 */
	public void windowClosing(WindowEvent e)
	{
		if (e.getSource()==main)
		{
      PilrcEdit.getPilrcEdit().quit();
 		}
    hide((Window)e.getSource());
	}

  public static void hide(Window w)
  {
    w.hide();
    visible.removeElement(w);
    CheckboxMenuItem cmi=(CheckboxMenuItem)viewMenu.get(w);
    if (cmi!=null)
      cmi.setState(false);
  }

	/**
	 * Invoked when a window has been closed.
	 */
	public void windowClosed(WindowEvent e) {}

	/**
	 * Invoked when a window is iconified.
	 */
	public void windowIconified(WindowEvent e) {}

	/**
	 * Invoked when a window is de-iconified.
	 */
	public void windowDeiconified(WindowEvent e) {}

	/**
	 * Invoked when a window is activated.
	 */
	public void windowActivated(WindowEvent e)
	{
		if ((System.currentTimeMillis()-deactivatedTime)<300)
		  return;
		Window w=(Window)e.getSource();
		visible.removeElement(w);
		visible.addElement(w);
		for (Enumeration en = visible.elements(); en.hasMoreElements() ;)
		{
			w=(Window)en.nextElement();
			w.show();
		}
		/*
		for (Enumeration en = windows.elements(); en.hasMoreElements() ;)
		{
			Object obj=en.nextElement();
			if (obj!=w&&obj instanceof Window)
				((Window)obj).show();
		}
		w.show();
		*/
  }

	/**
	 * Invoked when a window is de-activated.
	 */
	public void windowDeactivated(WindowEvent e)
	{
		deactivatedTime=System.currentTimeMillis();
	}

	static
	{
		StreamTokenizer st=null;
		try
		{
			FileReader fr=new FileReader(PREFS_FILE);
			st=new StreamTokenizer(new BufferedReader(fr));
			st.commentChar('#');
			st.ordinaryChar('(');
			st.ordinaryChar(')');
      st.wordChars(':',':');
      st.wordChars('/','/');
      st.wordChars('\\','\\');
      st.wordChars('_','_');
			while(st.nextToken()!=st.TT_EOF)
			{
				String name=st.sval;
				st.nextToken();
				if (st.ttype=='(')
				{
					st.nextToken();
					int x=(int)st.nval;
					st.nextToken();
					int y=(int)st.nval;
					st.nextToken();
					int width=(int)st.nval;
					st.nextToken();
					int height=(int)st.nval;
					st.nextToken(); // )
					windows.put(name,new Rectangle(x,y,width,height));
				}
				else
				if (st.ttype==st.TT_WORD)
					setPref(name,st.sval);
				else
				if (st.ttype==st.TT_NUMBER)
				  setPref(name,(int)st.nval);
			}
			fr.close();
		}
		catch(FileNotFoundException e)
	  {
		}
		catch(Exception e)
		{
      System.out.println("Error reading prefs file"+
        (st!=null?" at line "+st.lineno():"."));
		}
	}

  public static boolean getBooleanPref(String name)
  {
		return getBooleanPref(name,false);
	}

  public static boolean getBooleanPref(String name,boolean def)
  {
		if (!prefs.containsKey(name))
		{
			setPref(name,def);
		  return def;
		}
		return getPref(name).equalsIgnoreCase("true");
	}

  public static String getPref(String name,String def)
  {
		if (prefs.containsKey(name))
		  return getPref(name);
		setPref(name,def);
		return def;
	}

  public static int getIntPref(String name,int def)
  {
    if (name==null)
    {
      System.out.println("Name is null!");
      Debug.debug();
      System.exit(-1);

      return -1;
    }

		if (prefs.containsKey(name))
		  return getIntPref(name);
		setPref(name,def);
		return def;
	}

  public static String getPref(String name)
  {
    if (name==null)
    {
      System.out.println("Name is null!");
      return null;
    }

		return (String)prefs.get(name);
	}

  public static int getIntPref(String name)
  {
    if (name==null)
    {
      System.out.println("Name is null!");
      Debug.debug();
      System.exit(-1);

      return -1;
    }
    Integer i=(Integer)prefs.get(name);
    if (i==null)
    {
      System.out.println("Can't find preference! :"+name);
      Debug.debug(10);
      System.exit(-1);
    }
    return (i==null?-1:i.intValue());
	}

  public static void setPref(String name,String value)
  {
    prefs.put(name,value);
    notifyListeners(name);
	}

  public static void setPref(String name,int value)
  {
		prefs.put(name,new Integer(value));
    notifyListeners(name);
	}

  public static void setPref(String name,boolean value)
  {
		prefs.put(name,value?"True":"False");
    notifyListeners(name);
	}

  private static void notifyListeners(String s)
  {
    listeners.fireObjectChanged(s,listener,ChangeListener.MODIFIED,s);
  }

  public static void addChangeListener(String name,ChangeListener listener)
  {
    listeners.add(name,listener);
  }

  public static GeneralDialog getDialog()
  {
		if (main==null)
		  return null;
		if (dialog==null)
		  dialog=new GeneralDialog((Frame)main);
		return dialog;
	}
  public static void setMainFrame(Frame frame)
  {
		main=frame;
	}

  public static boolean registerFrame(Frame frame)
  {
		return registerFrame(frame.getTitle(),frame,false);
	}

  public static boolean registerFrame(Frame frame,boolean addToMenu)
  {
		return registerFrame(frame.getTitle(),frame,addToMenu);
	}

  public static boolean registerFrame(String name,Frame frame)
  {
		return registerFrame(name,frame,false);
	}

	public static boolean registerFrame(String name,Frame frame,boolean addToMenu)
	{
		Object o=windows.get(name);
		if (o instanceof Rectangle)
		{
			frame.setBounds((Rectangle)o);
		}
		windows.put(name,frame);
    if (icon==null)
      icon=BitmapLoader.loadImage("images/icon.bmp");
		frame.setIconImage(icon);
		frame.addWindowListener(listener);
		if (addToMenu)
		{
			CheckboxMenuItem cmi=new CheckboxMenuItem(name,frame.isVisible());
			cmi.addItemListener(listener);
		  viewMenu.put(frame,cmi);
		}
		return (o instanceof Rectangle);
	}

  public static void getViewMenuWindows(Menu menu)
  {
		for(Enumeration e=viewMenu.elements();e.hasMoreElements();)
		  menu.add((MenuItem)e.nextElement());
	}

	public static void save()
	{
		try
		{
			PrintWriter out=new PrintWriter(new FileWriter(PREFS_FILE));
			out.println("# General Preferences");
			for (Enumeration e = prefs.keys(); e.hasMoreElements() ;)
			{
				String name=(String)e.nextElement();
				Object value=prefs.get(name);
        if (name.indexOf(' ')!=-1)
          out.print("\""+name+"\" ");
        else
          out.print(name+" ");
				if (value instanceof String&&((String)value).indexOf(' ')!=-1)
				  out.println("\""+value+"\"");
				else
  				out.println(value);
			}
			out.println();
			out.println("# Window positions");
			for (Enumeration e = windows.keys() ; e.hasMoreElements() ;)
			{
				String name=(String)e.nextElement();
				Object obj=windows.get(name);
				Rectangle r=obj instanceof Window?((Window)obj).getBounds():(Rectangle)obj;
        if (name.indexOf(' ')!=-1)
          out.print("\""+name+"\" ");
        else
          out.print(name+" ");
				out.println(" ("+r.x+" "+r.y+" "+r.width+" "+r.height+")");
			}
			out.close();
		}
		catch(IOException e)
		{
			System.out.println(e);
		}
	}
}

