/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.awt.*;
import java.awt.event.*;

public class Screen extends Panel implements ChangeListener
{
	public static Rectangle fullScreen=new Rectangle(0,0,160,160);
  Image im;

  int zoom;
  Dimension size;
  Form form;
  Graphics ig;
  Image bim;
  boolean showBitmap=false;
  Editor editor;
  boolean refresh=false;
  Rectangle blatRect;
  Color bgColor;

  public Screen()
  {
		setZoom(Preferences.getIntPref("Zoom",2));
    objectChanged(null,0,null);
    Preferences.addChangeListener(Bitmap.PREF_BGCOLOR,this);
	}

  public void objectChanged(Object target,int type,Object data)
  {
    bgColor=new Color(Preferences.getIntPref(Bitmap.PREF_BGCOLOR));
    //setBackground(bgColor);
  }

  public int getZoom()
  {
		return zoom;
	}

  public void setZoom(int zoomFactor)
  {
		zoom=zoomFactor;
		size=new Dimension(160*zoom,160*zoom);
		if (editor!=null)
		  editor.zoomChanged(zoom);
		Preferences.setPref("Zoom",zoom);
	}

  //public void setColorDepth(int colorDepth)
  //{
  //  setBackground(new Color(Preferences.getIntPref(Bitmap.PREF_BGCOLOR)));
  //}

  public void setEditor(Editor editor)
  {
		this.editor=editor;
		editor.zoomChanged(zoom);
	}

  public void setBitmap(Image bim)
  {
		this.bim=bim;
	}

	public void toggleBitmap()
	{
		showBitmap=!showBitmap;
		repaint();
	}

  public void setForm(Form f)
  {
		refresh=true;
		form=f;
		if (editor!=null)
  		editor.setForm(f);
		repaint();
	}

  public Form getForm()
  {
		return form;
	}

  public Dimension getMinimumSize()
  {
		return size;
	}

	public Dimension getPreferredSize()
	{
		return size;
	}

  public void refresh()
  {
    refresh=true;
    repaint();
  }

  public void reblat(Rectangle rect)
  {
		repaint();
		blatRect=rect;
	}

  public void repaint()
  {
		blatRect=null;
		super.repaint();
	}

  public void update(Graphics g)
  {
		paint(g);
	}

  public void paint(Graphics g)
  {
		/*
		if (showBitmap)
		{
			g.drawImage(bim,0,0,160*zoom,160*zoom,0,0,160,160,null);
			return;
		}
*/
		if (form==null)
		{
		  g.clearRect(0,0,160*zoom,160*zoom);
		  return;
		}
		// get the modified region of the form
		Rectangle dirty=(blatRect!=null?blatRect:refresh?fullScreen:form.getDirtyRect());

		if (dirty==null) // if there is none, blat the full image to the screen
		{
			dirty=fullScreen;
		}
		else
		if (blatRect!=null)
		  blatRect=null;
		else
		{
			// redraw dirty part of form
			if (im==null)
			{
				im=createImage(160,160);
				ig=im.getGraphics();
			}
			ig.setClip(dirty.x,dirty.y,dirty.width,dirty.height);
			//ig.setColor(new Color((int)(Math.random()*256),(int)(Math.random()*256),(int)(Math.random()*256)));

			ig.setColor(bgColor);
			ig.fillRect(dirty.x,dirty.y,dirty.width,dirty.height);

      //ig.clearRect(dirty.x,dirty.y,dirty.width,dirty.height);
			ig.setColor(Color.black);
			if (form!=null)
				form.paint(ig);

			// add any points that the editor needs redrawn
			Rectangle editorDirty=editor.getDirty();
			if (editorDirty!=null)
  			dirty.add(editorDirty);

		}

		// make sure it is within the screen bounds
		if (dirty.x<0)
		  dirty.x=0;
		if (dirty.y<0)
		  dirty.y=0;
		if (dirty.x+dirty.width>160)
		  dirty.width=160-dirty.x;
		if (dirty.y+dirty.height>160)
		  dirty.height=160-dirty.y;

		// blat dirty section to screen
		g.drawImage(showBitmap?bim:im,dirty.x*zoom,dirty.y*zoom,(dirty.x+dirty.width)*zoom,(dirty.y+dirty.height)*zoom,
		  dirty.x,dirty.y,dirty.x+dirty.width,dirty.y+dirty.height,null);

		// clear the dirty settings in the form
		form.setDirty(null);
		refresh=false;

    editor.setDirty(null);

		// add any selection marks necessary
		editor.paint(g);
  }
}