/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.io.*;

public class StringResource extends Resource
{
  Project project;
  long lastModified;
  File file;
  protected String filename;
  protected String s;

  public StringResource(Project project,StreamTokenizer st) throws IOException
  {
    this.project=project;
    parse(st);
  }

  public StringResource(String s)
  {
    this.s=s;
  }

  public String getString()
  {
    if (file!=null)
      readFile();
    return s;
  }

  public void setString(String s)
  {
    this.s=s;
  }

  public void setFile(String s)
  {
    filename=s;
    file=new File(project.getParentDir(),filename);
    lastModified=0;
  }

  /**
   * For file based strings, read the string from the given file,
   * if it has been modified since last time we tried
   */
  private void readFile()
  {
    if (file.lastModified()!=lastModified)
    {
      try
      {
        BufferedReader br=new BufferedReader(new FileReader(file));
        String line=br.readLine();
        StringBuffer sb=new StringBuffer(line);
        while((line=br.readLine())!=null)
          sb.append('\n').append(line);
        s=sb.toString();
        lastModified=file.lastModified();
      }
      catch(IOException e)
      {
        e.printStackTrace();
      }
    }
  }

  public void parse(StreamTokenizer st) throws IOException
  {
    readID(st);
    s=readQuotedString(st);
    if (s==null) // wasn't a quoted string - must have been 'file'
    {
      st.nextToken();
      if (st.sval.equalsIgnoreCase("file"))
      {
        filename=readQuotedString(st);
        file=new File(project.getParentDir(),filename);
      }
    }
    else
    {
      st.nextToken();
      while (st.ttype!=st.TT_EOF&&st.sval.equals("\\"))
      {
        s+=readQuotedString(st);
        st.nextToken();
      }
      st.pushBack();
    }
  }

  public void write(PrintStream ps)
  {
    StringBuffer sb=new StringBuffer("STRING ID ");
    sb.append(id).append(' ');
    if (file!=null)
    {
      sb.append("FILE ").append(TextTools.quote(filename));
    }
    else
    {
      String[] split=TextTools.split(s,60,true);
      for(int i=0;i<split.length;i++)
      {
        sb.append(TextTools.quote(TextTools.escape(split[i])));
        if (i<split.length-1)
          sb.append("\\").append(Preferences.newLine).append("  ");
      }

    }
    ps.println(sb.toString());
  }
}