/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit.control;

import pilrcedit.*;
import java.awt.Graphics;
import java.awt.Color;
import java.io.*;

public class Checkbox extends FormObject
{
  public static final int CHECKED=0;
  public static final int GROUP=1;

  public static String PROP_CHECKED;
  public static String PROP_GROUP;

	private boolean checked;
  private Expression group=new Expression("");
  Font checkFont=Font.getFont(4);

  protected Object clone() throws CloneNotSupportedException
  {
		Checkbox c=(Checkbox)super.clone();
		c.group=new Expression(group);
		return c;
	}

  public void getProperties(Properties prop)
  {
		super.getProperties(prop);
    Keywords.setDefault(this);
		prop.addProperty(PROP_CHECKED,boolStrings,checked?0:1);
		prop.addProperty(PROP_GROUP,group.getExpression());
	}

  public boolean setProperty(String name,String value,int index)
  {
    if (name==PROP_CHECKED)
      checked=(index==0);
    else
    if (name==PROP_GROUP)
      group.setExpression(value);
    else
      return super.setProperty(name,value,index);
		evaluatePosition();
		return true;
	}

  protected boolean parseOption(StreamTokenizer st) throws IOException
  {
		if (super.parseOption(st))
		  return true;
    int key=Keywords.get(this,st.sval);
    switch(key)
    {
      case CHECKED: checked=true; break;
      case GROUP: group.setExpression(readNormalString(st)); break;
      default: return false;
		}
		return true;
  }

	protected void writeOptions(StringBuffer sb)
	{
		super.writeOptions(sb);
		if (checked)
		  sb.append(' ').append(Keywords.get(this,CHECKED));
		if (!group.isEmpty())
  		sb.append(' ').append(Keywords.get(this,GROUP)).append(' ').append(group);
	}

  public int autoWidth()
  {
		return super.autoWidth()+18;
	}

  public void paint(Graphics g)
  {
		int x1=x;
		int y1=y;
		int x2=x1+width-1;
		int y2=y1+height-1;
		g.setColor(usable?Color.black:Color.lightGray);
		checkFont.drawChar(g,(char)(checked?1:0),x,y);
		getFont().drawString(g,TextTools.unescape(label),x+17,y);
		g.setColor(Color.black);
  }
}