/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit.control;

import pilrcedit.*;
import java.awt.*;
import java.io.*;
import java.util.Enumeration;

public class FormBitmap extends FormObject implements ChangeListener
{
  public static final int BITMAP=0;

  public static String PROP_BITMAP;
  public static String NEW_BITMAP;


	String bmpString;
	int bmpID;
	Image im;
  ResourceCollection rc;
  Bitmap bitmap;
  boolean newBitmap=false;

  public FormBitmap()
  {
    Preferences.addChangeListener(Bitmap.PREF_COLOR_DEPTH,this);
  }

  public void objectChanged(Object source,int id,Object data)
  {
    if (source==rc&&props!=null)
    {
      props.refresh();
    }
    else
    if (source==bitmap&&id==Bitmap.IMAGE_CHANGED)
    {
      im=null;
      //fetchImage();
      evaluatePosition();
      form.repaint();
    }
    else
    if (source instanceof Preferences)
      im=null;
  }

  public boolean hasWidthAndHeight()
  {
		return false;
	}

  public boolean hasLabel()
  {
		return false;
	}

	public void parse(StreamTokenizer st) throws IOException
	{
    readLocation(st);
    parseOptions(st);
    evaluatePosition();
	}

  public int autoWidth()
  {
    if (im==null)
      if (!fetchImage())
        return 10;
    return im.getWidth(null);
	}

	public int autoHeight()
	{
    if (im==null)
      if (!fetchImage())
        return 10;
    return im.getHeight(null);
	}


  protected boolean parseOption(StreamTokenizer st) throws IOException
  {
		if (super.parseOption(st))
		  return true;
		if (Keywords.get(this,st.sval)==BITMAP)
		{

			bmpString=parseID(st);
      bmpID=Expression.evaluate(bmpString);
		}
		else
		  return false;
		return true;
  }

  /**
   * Sets the properties of this object into the given Properties object
   * using the prop.addProperty() methods.
   */
  public void getProperties(Properties prop)
  {
		props=prop;
		prop.addProperty(PROP_LEFT,xExp.getExpression());
		prop.addProperty(PROP_TOP,yExp.getExpression());
		prop.addProperty(PROP_USABLE,boolStrings,usable?0:1);
		prop.addProperty(PROP_DISABLED,boolStrings,disabled?0:1);
    rc=(ResourceCollection)getProject().getBitmaps();
    rc.addChangeListener(this);
    String[] bitmaps=new String[rc.size()-1];
    int select=-1;
    int i=0;
    bitmaps[i++]=NEW_BITMAP;
    Enumeration e=rc.elements();
    e.nextElement();
    e.nextElement();
    if (!newBitmap&&bitmap==null&&bmpID!=-1)
      setBitmap(getProject().getBitmap(bmpID));
    while(e.hasMoreElements())
    {
      Bitmap b=(Bitmap)e.nextElement();
      if ((newBitmap&&i==bitmaps.length-1)||(!newBitmap&&b==bitmap))
      {
        select=i;
        if (newBitmap)
        {
          newBitmap=false;
          setBitmap(b);
        }
      }
      bitmaps[i++]=b.getID();
    }
    if (select==-1) // if there are no bitmaps selected
    {
      if (bitmaps.length==1) // no bitmaps so open bitmap frame
      {
        select=0;
        newBitmap=true;
      }
      else // otherwise, choose the first
      {
        select=1;
        setBitmap((Bitmap)rc.getResourceAt(2));
      }
    }
    prop.addProperty(PROP_BITMAP,bitmaps,select,true);
    if (newBitmap)
      PilrcEdit.getPilrcEdit().openBitmapFrame(true);
	}

  public void propertiesClosed()
  {
    if (rc!=null)
      rc.removeChangeListener(this);
    super.propertiesClosed();
  }

  public boolean setProperty(String name,String value,int index)
  {
    if (name==PROP_BITMAP)
    {
      if (index==0)
      {
        newBitmap=true;
        PilrcEdit.getPilrcEdit().openBitmapFrame(true);
      }
      else
      {
        setBitmap((Bitmap)rc.getResourceAt(index+1));
      }
    }
    else
      return super.setProperty(name,value,index);
    return true;
  }

  private void setBitmap(Bitmap b)
  {
    if (b==null||b==bitmap)
      return;
    if (bitmap!=null)
      bitmap.removeChangeListener(this);
    bitmap=b;
    bitmap.addChangeListener(this);
    im=null;
    evaluatePosition();
  }

  private boolean fetchImage()
  {
    if (bitmap!=null)
      im=bitmap.getImage();
    else
    if (bmpID!=-1)
    {
      if (bmpID==0)
        setBitmap((Bitmap)getProject().getBitmaps().getResourceAt(2));
      else
        setBitmap(getProject().getBitmap(bmpID));
      if (bitmap!=null)
        im=bitmap.getImage();
      if (im==null&&!getProject().isParsing())
      {
        bmpID=-1;
        return false;
      }
    }
    else
      return false;

    return im!=null;
    /*
    if (im!=null)
    {
      setDimensions(null,null,""+im.getWidth(null),""+im.getHeight(null));
      return true;
    }
    else
      return false;
    */
  }

  public void paint(Graphics g)
  {
    if (im==null&&bmpID!=-1)
      fetchImage();
    if (im!=null)
      g.drawImage(im,x,y,null);
    else
    {
      g.drawRect(x,y,width,height);
      g.drawLine(x,y,x+width,y+height);
      g.drawLine(x+width,y,x,y+height);
    }
  }

  public void doubleClicked()
  {
    PilrcEdit.getPilrcEdit().openBitmapFrame(bitmap,false);
  }

  public void write(PrintStream ps)
  {
		StringBuffer sb=new StringBuffer();
    Keywords.setDefault(this);
		sb.append(Keywords.get(NAME));
		sb.append(' ').append(Keywords.get(AT)).append(" (").append(xExp).append(' ').append(yExp);
		sb.append(") ").append(Keywords.get(BITMAP)).append(' ');
    if (bitmap==null)
      sb.append(bmpString);
    else
      sb.append(bitmap.getID());
		writeOptions(sb);
		ps.println(sb.toString());
	}
}