/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit.control;

import pilrcedit.*;
import java.awt.*;
import java.io.*;
import java.util.Vector;

public class PopupTrigger extends FormObject
{
  public static String PROP_POPPED_UP;
  public static String REMOVE_LIST;
  public static String POPUP_LIST;
  public static String CREATE_LIST;
	private List list;

  public boolean isVisible()
  {
		return list==null||!list.isVisible();
	}

  public void getPopupMenuItems(Vector v)
  {
		super.getPopupMenuItems(v);
		v.addElement("-");
		if (list!=null)
		{
			v.addElement(REMOVE_LIST);
  		v.addElement(POPUP_LIST);
		}
    else
			v.addElement(CREATE_LIST);
	}

	public boolean popupMenuItemSelected(String s)
	{
		if (s==REMOVE_LIST)
		{
			form.remove(list);
			list=null;
		}
		if (s==POPUP_LIST)
			list.popUp();
		else
		if (s==CREATE_LIST)
		{
			List list=new List();
			form.insert(list,this);
			list.setDimensions(
        Keywords.get(PreviousExpression.class,PreviousExpression.PREVLEFT),
        Keywords.get(PreviousExpression.class,PreviousExpression.PREVTOP),
        Keywords.get(DimensionExpression.class,DimensionExpression.AUTO),
        Keywords.get(DimensionExpression.class,DimensionExpression.AUTO));
			setList(list);
			list.popUp();
		}
		else
      return super.popupMenuItemSelected(s);
    return true;
	}

  public void doubleClicked()
  {
		if (list!=null)
		  list.popUp();
	}

  public void getProperties(Properties prop)
  {
		super.getProperties(prop);
		if (list!=null)
  		prop.addProperty(PROP_POPPED_UP,boolStrings,1);
	}

  public boolean setProperty(String name,String value,int index)
  {
		if (name==PROP_POPPED_UP)
			list.popUp();
		else
			return super.setProperty(name,value,index);
		evaluatePosition();
		return true;
	}

  public int autoWidth()
  {
		return super.autoWidth()+18;
	}

  public void setList(List list)
  {
		this.list=list;
		if (list!=null)
		{
			list.setPopupTrigger(this);
			if (label==null||label.equals(""))
				label=list.getSelected();
		}
	}

  public void paint(Graphics g)
  {
		getFont().drawString(g,TextTools.unescape(label),x+15,y);
		int x=this.x+3;
		int y=this.y+4;
		g.drawLine(x,y,x+7,y);
		g.drawLine(x+1,y+1,x+6,y+1);
    g.drawLine(x+2,y+2,x+5,y+2);
    g.drawLine(x+3,y+3,x+4,y+3);
  }

  public void write(PrintStream ps)
  {
		super.write(ps);
		if (list!=null)
			ps.println("  "+Keywords.get(form,form.POPUPLIST)+" "+id+" "+list.getID());
	}
}