/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit.control;

import pilrcedit.*;
import java.awt.Graphics;
import java.awt.Color;
import java.io.*;

public class Pushbutton extends FormObject
{
  public static final int GROUP=0;

  public static String PROP_GROUP;

	private Expression group=new Expression("");

  protected boolean parseOption(StreamTokenizer st) throws IOException
  {
		if (super.parseOption(st))
		  return true;
		if (Keywords.get(this,st.sval)==GROUP)
		{
			st.lowerCaseMode(false);
			st.nextToken();
			st.lowerCaseMode(true);
			group.setExpression(st.sval);
		}
		else
		  return false;
		return true;
  }

	protected void writeOptions(StringBuffer sb)
	{
		super.writeOptions(sb);
		if (!group.isEmpty())
		  sb.append(' ').append(Keywords.get(this,GROUP)).append(' ').append(group);
	}

  public void getProperties(Properties prop)
  {
		super.getProperties(prop);
		prop.addProperty(PROP_GROUP,group.getExpression());
	}

  public boolean setProperty(String name,String value,int index)
  {
		if (name==PROP_GROUP)
	  {
			group.setExpression(value);
	  }
		else
		  return super.setProperty(name,value,index);
		evaluatePosition();
		return true;
	}

  public int autoWidth()
  {
		return super.autoWidth()+6;
	}

  public void paint(Graphics g)
  {
		Font font=getFont();
    g.setColor(usable?Color.black:Color.lightGray);
		String displayLabel=TextTools.unescape(label);
		font.drawString(g,displayLabel,x+(width-font.stringWidth(displayLabel)+1)/2,
		  y+(height-font.getHeight())/2);
    g.drawRect(x-1,y-1,width+1,height+1);
		g.setColor(Color.black);
  }
}