/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit.control;

import pilrcedit.*;
import java.awt.*;
import java.io.*;

public class ScrollBar extends FormObject
{
  public static final int VALUE=0;
  public static final int MIN=1;
  public static final int MAX=2;
  public static final int PAGESIZE=3;

  public static String PROP_VALUE;
  public static String PROP_MIN;
  public static String PROP_MAX;
  public static String PROP_PAGESIZE;

	Expression min=new Expression(0);
	Expression max=new Expression(100);
	Expression value=new Expression(0);
	Expression page=new Expression(10);

  protected Object clone() throws CloneNotSupportedException
  {
		ScrollBar s=(ScrollBar)super.clone();
		s.min=new Expression(min);
		s.max=new Expression(max);
		s.value=new Expression(value);
		s.page=new Expression(page);
		return s;
	}

  public boolean hasLabel()
  {
		return false;
	}

  public int autoWidth()
  {
		return 7;
	}

  public int autoHeight()
  {
		return 100;
	}

  public void getProperties(Properties prop)
  {
		super.getProperties(prop);
		prop.addProperty(PROP_VALUE,value.getExpression());
		prop.addProperty(PROP_MIN,min.getExpression());
		prop.addProperty(PROP_MAX,max.getExpression());
		prop.addProperty(PROP_PAGESIZE,page.getExpression());
	}

  public boolean setProperty(String name,String value,int index)
  {
		try
		{
			if (name==PROP_MIN)
				min.setExpression(value);
			else
			if (name==PROP_MAX)
			  max.setExpression(value);
			else
			if (name==PROP_VALUE)
			  this.value.setExpression(value);
			else
			if (name==PROP_PAGESIZE)
			  page.setExpression(value);
			else
				return super.setProperty(name,value,index);
			evaluatePosition();
			return true;
		}
		catch(NumberFormatException e)
		{
			throw(new IllegalArgumentException());
		}
	}

  protected boolean parseOption(StreamTokenizer st) throws IOException
  {
		if (super.parseOption(st))
		  return true;
    int key=Keywords.get(this,st.sval);
    switch(key)
    {
      case VALUE: value.setExpression(readNormalString(st)); break;
      case MIN: min.setExpression(readNormalString(st)); break;
      case MAX: max.setExpression(readNormalString(st)); break;
      case PAGESIZE: page.setExpression(readNormalString(st)); break;
      default: return false;
    }
		return true;
  }

	protected void writeOptions(StringBuffer sb)
	{
		super.writeOptions(sb);
    Keywords.setDefault(this);
		sb.append(' ').append(Keywords.get(VALUE)).append(' ').append(value);
		sb.append(' ').append(Keywords.get(MIN)).append(' ').append(min);
    sb.append(' ').append(Keywords.get(MAX)).append(' ').append(value);
    sb.append(' ').append(Keywords.get(PAGESIZE)).append(' ').append(min);
	}

  public void setDimensions(String x,String y,String width,String height)
  {
		if (width!=null&&width.equals(Keywords.get(widthExp,widthExp.AUTO)))
		  width=""+autoWidth();
		if (height!=null&&height.equals(Keywords.get(heightExp,heightExp.AUTO)))
		  height=""+autoHeight();
		super.setDimensions(x,y,width,height);
	}

  public void paint(Graphics g)
  {
		g.setColor(usable?Color.black:Color.lightGray);
		int w=(width/2)*2+1;
		int lines=w/2+1;
		for(int i=0;i<lines;i++)
		{
		  g.drawLine(x+lines-i-1,y+i+1,x+lines+i-1,y+i+1);
		  g.drawLine(x+lines-i-1,y+height-i-2,x+lines+i-1,y+height-i-2);
		}
		int barheight=height-lines*2-4;
		int maxv=max.getValue();
		int minv=min.getValue();
		if (minv==maxv)
		  maxv++;
		int pagesize=page.getValue();
		int cabHeight=barheight*pagesize/(maxv-minv);
		int cabStart=value.getValue()*(barheight-cabHeight)/(maxv-minv);
		int cabEnd=cabStart+cabHeight-1;
		for(int j=0;j<barheight;j++)
		{
		  int yd=y+lines+2+j;
		  if (j>=cabStart&&j<=cabEnd)
		    g.drawLine(x+2,yd,x+width-3,yd);
		  else
		  if (j!=cabStart-1&&j!=cabEnd+1)
    		for(int i=2+j%2;i<width-2;i+=2)
    		  g.drawLine(x+i,yd,x+i,yd);
		}
		g.setColor(Color.black);
  }
}