/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit.java2;

import pilrcedit.*;
import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.io.*;

public class DropManager implements DropTargetListener
{
  DropTarget dropTarget;
  pilrcedit.PilrcEdit pe;

  public DropManager(Component c,pilrcedit.PilrcEdit pe)
  {
    dropTarget= new DropTarget (c, this);
    this.pe=pe;
  }

  public void dragEnter (DropTargetDragEvent dropTargetDragEvent)
  {
    dropTargetDragEvent.acceptDrag (DnDConstants.ACTION_COPY);
  }

  public void dragExit (DropTargetEvent dropTargetEvent) {}
  public void dragOver (DropTargetDragEvent dropTargetDragEvent) {}
  public void dropActionChanged (DropTargetDragEvent dropTargetDragEvent){}

  public synchronized void drop (DropTargetDropEvent dropTargetDropEvent)
  {
    try
    {
      Transferable tr = dropTargetDropEvent.getTransferable();
      int x=(int)dropTargetDropEvent.getLocation().getX();
      int y=(int)dropTargetDropEvent.getLocation().getY();
      if (tr.isDataFlavorSupported (DataFlavor.javaFileListFlavor))
      {
        dropTargetDropEvent.acceptDrop(DnDConstants.ACTION_COPY);
        java.util.List fileList = (java.util.List)
            tr.getTransferData(DataFlavor.javaFileListFlavor);
        File file=(File)fileList.get(0);
        //System.out.println(file);
          pe.open(file);
        dropTargetDropEvent.getDropTargetContext().dropComplete(true);
        return;
      }
      else
      if (tr.isDataFlavorSupported(DataFlavor.stringFlavor))
      {
        dropTargetDropEvent.acceptDrop(DnDConstants.ACTION_COPY);
        String s=(String)tr.getTransferData(DataFlavor.stringFlavor);
        //System.out.println("st>"+s+"<");
        pe.getEditor().processExternalInput(s,x,y);
        dropTargetDropEvent.getDropTargetContext().dropComplete(true);
        return;
      }
      else
      {
        DataFlavor[] flavors=tr.getTransferDataFlavors();
        int i;
        boolean unicode=false;
        for(i=0;i<flavors.length;i++)
        {
          //System.out.println(flavors[i].getMimeType());
          if (flavors[i].getMimeType().startsWith("text/plain")&&
              InputStream.class.isAssignableFrom(flavors[i].getRepresentationClass()))
          {
            unicode=flavors[i].getMimeType().endsWith("unicode");
            break;
          }
        }
        if (i<flavors.length)
        {
          dropTargetDropEvent.acceptDrop(DnDConstants.ACTION_COPY);//_OR_MOVE);
          InputStream is=(InputStream)tr.getTransferData(flavors[i]);
          InputStreamReader isr=null;
          if (unicode)
            isr=new InputStreamReader(is,"Unicode");
          else
            isr=new InputStreamReader(is);
          StringBuffer str = new StringBuffer();
          int in=-1;
          while((in = isr.read()) > 0 )
            str.append((char)in);

          while(isr.read()!=-1); // read the rest
          //System.out.println("is!"+str.toString()+"!");
          pe.getEditor().processExternalInput(str.toString(),x,y);
          dropTargetDropEvent.getDropTargetContext().dropComplete(true);
          return;
        }
      }
    }
    catch (Exception io)
    {
      io.printStackTrace();
    }
    //System.out.println("Rejected");
    dropTargetDropEvent.rejectDrop();
  }
}